function [B,A,T,varY] = JoJoBoxJenkins(F,U,Y,nb,na,feedback,fix)
% 
% Identification of a linear dynamic system in the frequency domain
%     for arbitrary excitations using a nonparametric noise model
%     and a discrete time plant model
%
% F : frequency vector (in Hz)
% U : input Fourier coefficients
% Y : output Fourier coefficients
% nb, na: order numerator and denominator
% fix: if fix='yes': z^0 in numerator is fixed to zero, else it is free
%
% B: numerator parameters
% A: denominator parameters
% varY: nonparametric noise model: the complex variance of the 
%                                     output at the freq. F
%
% Johan Schoukens, 6 March 1998 (a rainy afternoon)

if nargin <7, fix='no';end % default b0 not fixed to zero
if nargin <6, feedback='of';end  % check for identification under feedback

% initial estimate of the noise model, no system parameters used
varY=varestJoJo([F U Y]);  % first rough initial noise model without using model knowledge

[pvectz]=elis(fiddata(Y,U,F,varY,zeros(size(F))),'z',nb,na,struct('fs',1)); % initialization without transients
[domain,B,A,delay,fs,Znum,Zdenom,comments,fdate,T]=imppar(pvectz);

% improved estimate, no transient information used in noise estimate
varY=varestJoJo([F U Y],B,A);  % 2nd rough initial noise model without transient knowledge
[pvectz]=elis(fiddata(Y,U,F,varY,zeros(size(F))),'z',nb,na,struct('fs',1,'transients','on','initmodel',fidmodel('z^-1',B,A,0,1)));
[domain,B,A,delay,fs,Znum,Zdenom,comments,fdate,T]=imppar(pvectz);

% full estimate, including the transient estimate in noise estimate
loop=1;   % loop: number of updates of the nonparametric noise model
for k=1:loop  % improved estimate using the robust noise variance estimates
varY=varestJoJo([F U Y],B,A,T);
[pvectz]=elis(fiddata(Y,U,F,varY,zeros(size(F))),'z',nb,na,struct('fs',1,'transients','on','initmodel',fidmodel('z^-1',B,A,0,1)));
[domain,B,A,delay,fs,Znum,Zdenom,comments,fdate,T]=imppar(pvectz);
end

% refine if feedback present
if feedback == 'on'  % refine the estimates for feedback
	for k=1:loop*2 
	varY=varestJoJo([F U Y],B,A,T,'on');
	[pvectz]=elis(fiddata(Y,U,F,varY,zeros(size(F))),'z',nb,na,struct('fs',1,'transients','on','initmodel',fidmodel('z^-1',B,A,0,1)));
	[domain,B,A,delay,fs,Znum,Zdenom,comments,fdate,T]=imppar(pvectz);
	end
end
