function Theta = ExtractParam(DeltaParam, OldTheta, Sel);
%
% function Theta = ExtractParam(DeltaParam, OldTheta, Sel);
%
%		Extracts the parameter update Theta from the variation of the
%		free parameters in DeltaParam
%
%	Output parameter
%
%		Theta		=	new estimate plant, noise, and initial conditions parameters
%						structure with fields 'A', 'B', 'Ig', 'C', 'D', 'Ih'
%							Theta = struct('A',[],'B',[], 'Ig', [], 'C',[],'D',[], 'Ih', [])
%							Theta.A = 1 x (OrderA+1)
%								Theta.A(r) = coefficient a(r-1) of Omega^(r-1) 
%							Theta.B = ny x nu x (OrderB+1)
%								Theta.B(i,j,r) = coefficient b(i,j,r-1) of Omega^(r-1)
%							Theta.Ig = ny x (OrderIg+1)
%								Theta.Ig(i,r) = coefficient ig(i,r-1) of Omega^(r-1) 
%							Theta.C = ny x ny x (OrderA+1)
%								Theta.C(i,j,r) = coefficient c(i,j,r-1) of Omega^(r-1) 
%							Theta.D = 1 x (OrderD+1)
%								Theta.D(r) = coefficient d(r-1) of Omega^(r-1)
%							Theta.Ih = ny x (OrderIh+1)
%								Theta.Ih(i,r) = coefficient ih(i,r-1) of Omega^(r-1)
%
%	Input parameter
%
%		DeltaParam	=	column vector containing the free model parameters,
%						dimension: number of free parameters x 1
%		OldTheta	=	previous estimate plant, noise, and initial conditions parameters
%						structure with fields 'A', 'B', 'Ig', 'C', 'D', 'Ih'
%						OldTheta = struct('A',[],'B',[], 'Ig', [], 'C',[],'D',[], 'Ih', [])
%							OldTheta.A = 1 x (OrderA+1)
%								OldTheta.A(r) = coefficient a(r-1) of Omega^(r-1) 
%							OldTheta.B = ny x nu x (OrderB+1)
%								OldTheta.B(i,j,r) = coefficient b(i,j,r-1) of Omega^(r-1)
%							OldTheta.Ig = ny x (OrderIg+1)
%								OldTheta.Ig(i,r) = coefficient ig(i,r-1) of Omega^(r-1) 
%							OldTheta.C = ny x ny x (OrderA+1)
%								OldTheta.C(i,j,r) = coefficient c(i,j,r-1) of Omega^(r-1) 
%							OldTheta.D = 1 x (OrderD+1)
%								OldTheta.D(r) = coefficient d(r-1) of Omega^(r-1)
%							OldTheta.Ih = ny x (OrderIh+1)
%								OldTheta.Ih(i,r) = coefficient ih(i,r-1) of Omega^(r-1)
%		Sel			=	structure with fields 'A', 'B', 'Ig', 'C', 'D', 'Ih'
%							Sel = struct('A',[],'B',[], 'Ig', [], 'C',[],'D',[], 'Ih', [])
%							Sel.A = 1 x (OrderA+1)
%								Sel.A(r) = 1 if coeff. a(r-1) is unknown
%								Sel.A(r) = 0 if coeff. a(r-1) = 0
%							Sel.B = ny x nu x (OrderB+1)
%								Sel.B(i,j,r) = 1 if coeff. b(i,j,r-1) is unknown
%								Sel.B(i,j,r) = 0 if coeff. b(i,j,r-1) = 0
%							Sel.Ig = ny x (OrderIg+1)
%								Sel.Ig(i,r) = 1 if coeff. ig(i,r-1) is unknown
%								Sel.Ig(i,r) = 0 if coeff. ig(i,r-1) = 0
%							Sel.C = ny x ny x (OrderC+1)
%								Sel.C(i,j,r) = 1 if coeff. c(i,j,r-1) is unknown
%								Sel.C(i,j,r) = 0 if coeff. c(i,j,r-1) = 0
%							Sel.D = 1 x (OrderD+1)
%								Sel.D(r) = 1 if coeff. d(r-1) is unknown
%								Sel.D(r) = 0 if coeff. d(r-1) = 0
%							Sel.Ih = ny x (OrderIh+1)
%								Sel.Ih(i,r) = 1 if coeff. ih(i,r-1) is unknown
%								Sel.Ih(i,r) = 0 if coeff. ih(i,r-1) = 0
%
%
% Copyright (c) Rik Pintelon, Vrije Universiteit Brussel - dept. ELEC, December 2004 
% All rights reserved.
% Software can be used freely for non-commercial applications only.
%

nu = size(OldTheta.B,2);
ny = size(OldTheta.B,1);

% coefficients polynomial A
sumA = sum(Sel.A);
DeltaA = zeros(1,length(Sel.A));
lower = 1;
upper = sumA;
DeltaA(Sel.A == 1) = DeltaParam(lower:upper).';
Theta.A = OldTheta.A + DeltaA;

% coefficients ny x nu matrix polynomial B
Theta.B = zeros(size(OldTheta.B));
for jj = 1:nu
	for ii = 1:ny
		sumB = sum(Sel.B(ii,jj,:));
		DeltaB = zeros(1, 1, length(Sel.B(ii,jj,:)));
		lower = upper + 1;
		upper = lower + sumB - 1;
		DeltaB(1, 1, Sel.B(ii,jj,:) == 1) = DeltaParam(lower:upper).';
		Theta.B(ii,jj,:) = OldTheta.B(ii,jj,:) + DeltaB;
	end % jj
end % ii

% coefficients ny x 1 vector polynomial Ig
Theta.Ig = zeros(size(OldTheta.Ig));
for ii = 1:ny
	sumIg = sum(Sel.Ig(ii,:));
	DeltaIg = zeros(1, length(Sel.Ig(ii,:)));
	lower = upper + 1;
	upper = lower + sumIg - 1;
	DeltaIg(Sel.Ig(ii,:) == 1) = DeltaParam(lower:upper).';
	Theta.Ig(ii,:) = OldTheta.Ig(ii,:) + DeltaIg;
end % ii

% coefficients ny x ny matrix polynomial C
Theta.C = zeros(size(OldTheta.C));
for jj = 1:ny
	for ii = 1:ny
		sumC = sum(Sel.C(ii,jj,:));
		DeltaC = zeros(1, 1, length(Sel.C(ii,jj,:)));
		lower = upper + 1;
		upper = lower + sumC - 1;
		DeltaC(1, 1, Sel.C(ii,jj,:) == 1) = DeltaParam(lower:upper).';
		Theta.C(ii,jj,:) = OldTheta.C(ii,jj,:) + DeltaC;
	end % jj
end % ii

% coefficients polynomial D
sumD = sum(Sel.D);
DeltaD = zeros(1,length(Sel.D));
lower = upper + 1;
upper = lower + sumD - 1;
DeltaD(Sel.D == 1) = DeltaParam(lower:upper).';
Theta.D = OldTheta.D + DeltaD;

% coefficients ny x 1 vector polynomial Ih
Theta.Ih = zeros(size(OldTheta.Ih));
for ii = 1:ny
	sumIh = sum(Sel.Ih(ii,:));
	DeltaIh = zeros(1,length(Sel.Ih(ii,:)));
	lower = upper + 1;
	upper = lower + sumIh - 1;
	DeltaIh(Sel.Ih(ii,:) == 1) = DeltaParam(lower:upper).';
	Theta.Ih(ii,:) = OldTheta.Ih(ii,:) + DeltaIh;
end % ii
