%
% Discrete-time or continuous-time OE modeling starting from frequency domain data.
% This example uses the "MIMO_MaximumLikelihood" toolbox
%
% Rik Pintelon, 21 October 2011
%

clear all
close all


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Definition simulation parameters %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

N = 5000;               % number of time domain samples
PlantPlane = 'z';       % discrete-time plant
PlantPlane = 's';       % continuous-time plant
% PlantPlane = 'w';       % sqrt(s)-domain
RecipPlant = 0;         % if 1 then reciprocal plant transfer function

stde = 0.01;            % standard deviation output noise
Ts = 1/5.5;             % sampling period
fs = 1/Ts;              % sampling frequency

fmin = 0.1;     % begin frequency band of interest
fmax = 2;       % end frequency band of interest

% selection of the corresponding DFT lines
Select = [floor(fmin/(fs/N))+1:1:floor(fmax/(fs/N))+1].';	% select from fmin to fmax
F = length(Select);                                         % number of frequencies in the frequency band of interest
SelectAll = [1:1:N+1].';                                    % from DC to Nyquist on 2*N points for s-domain
freq = (Select-1)*fs/N;                                     % frequencies selected DFT lines
freqAll = ((SelectAll-1)/(2*N)/Ts);                         % frequencies all DFT lines
if PlantPlane == 's'
    sAll = sqrt(-1)*2*pi*freqAll;
end % if s-domain
if PlantPlane == 'w'
    sAll = (sqrt(-1)*2*pi*freqAll).^0.5;
end % if s-domain

nu = 2;                 % number of inputs  nu <= 3
ny = 3;                 % number of outputs ny <= 3

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Definition plant model transfer function %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

nmax = 3;
switch PlantPlane

    case 'z'
        b0 = rand(nmax, nmax, 5);
        % chebychev polynomial for noise model
        [b110, a0] = cheby1(4, 3, 0.5);

    case 's'
        b0 = rand(nmax, nmax, 3);
        % inverse chebychev polynomial for plant
        [b110, a0] = cheby2(3, 40, 2*pi,'s');
        b110 = fliplr(b110(2:end));
        b110(2) = 5e-4;
        a0 = fliplr(a0);
        b0(1,1,:) = b110;
        b0(2,2,:) = fliplr([a0(1) 2e-4 2e-3]);
        b0(3,3,:) = fliplr([2e-3 a0(1) 2e-4]);
        b0(3,2,:) = fliplr([5e-3 5e-4 a0(1)]);
        
	case 'w'
        rand('state',0);
		b0 = rand(nmax, nmax, 4);
		% inverse chebychev polynomial for plant
		[b110, a0] = cheby2(4, 40, 2*2*pi,'s');
		b110 = fliplr(b110(2:end));
		b110(2) = 5e-4;
        ThePoles = roots(a0);
        ThePoles = ThePoles.^0.5;
        a0 = poly(ThePoles);
		a0 = fliplr(a0);
		b0(1,1,:) = 0.1*b110;
        b0(1,2,:) = b0(1,1,:);
        b0(1,3,:) = b0(1,2,:);
        b0(2,1,:) = b0(2,1,:);
		b0(2,2,:) = fliplr([a0(1) 0 2e-3 1])/100;
		b0(3,3,:) = fliplr([2e-3 a0(1) 0 1]);
		b0(3,2,:) = fliplr([0 5e-4 a0(1) 1]);

end % switch

na = length(a0) - 1;
nb = size(b0,3) - 1;
nc = 0;
nd = 0;

% if reciprocal == 1 then make plant transfer function reciprocal
if RecipPlant
	for ll = 1:nb+1
		b0(:,:,ll) = b0(:,:,ll) + b0(:,:,ll).';
	end
end

% reduce to ny outputs and nu inputs
b0 = b0(1:ny, 1:nu, :);


%%%%%%%%%%%%%%%%%%%%%%%%%%
% Set the default values %
%%%%%%%%%%%%%%%%%%%%%%%%%%

% create the structures required for the MIMO_BoxJenkins function
[Sel, Theta0, ModelVar, IterVar] = DefaultValues(na, nb, [], [], nu, ny, PlantPlane, [], 'OE', RecipPlant);

% s-domain: plant transient term needs a higher order
if (PlantPlane == 's') || (PlantPlane == 'w')
    nig = 4;
    ModelVar.nig = nig;
    Sel.Ig = ones(ny, nig+1);
    Theta0.Ig = zeros(ny, nig+1);
end % if s-domain

% true parameter values (except the plant transient parameters)
Theta0.A = a0;
for ii = 1:ny
	for jj = 1:nu
		Theta0.B(ii,jj,:) = b0(ii,jj,:);
	end % jj
end % ii


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Calculation noisy outputs %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

Te = fliplr(hankel(ones(ny,1)));
switch PlantPlane
    case 'z'                % calculation of N samples of the time domain response
        u = randn(nu, N);
        y0 = zeros(ny, N);
        for jj=1:ny
            for ii=1:nu
                y0(jj,:) = y0(jj,:) + filter(squeeze(b0(jj,ii,:)),a0,u(ii,:));
            end % ii
        end % jj
        e = randn(ny, N)*stde;
        e = Te*e;
        y = y0 + e;
        
    case {'s','w'}           % calculation 2*N samples time response via the frequency domain
        u = randn(nu, 2*N);
        U = fft(u,[],2);
        Y0 = zeros(ny, length(SelectAll));
        for jj=1:ny
            for ii=1:nu
                Y0(jj,:) = Y0(jj,:) + ((polyval(fliplr(squeeze(b0(jj,ii,:)).'),sAll)./polyval(fliplr(a0),sAll)).').*U(ii,SelectAll);
            end % ii
        end % jj
        y0 = 2*real(ifft([zeros(ny,1),Y0(:,2:end-1),zeros(ny,N)],[],2));
        u = u(:,1:N);
        y0 = y0(:,1:N);
        e = randn(ny, N)*stde;
        e = Te*e;
        y = y0 + e;
       
end % switch


%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Data for identification %
%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Selection of the frequency band of interest
U = fft(u,[],2)/sqrt(N);
Y = fft(y,[],2)/sqrt(N);
U = U(:, Select);
Y = Y(:, Select);

% filling out the data structure 
data.U = U;
data.Y = Y;
data.freq = freq;
data.Ts = Ts;


%%%%%%%%%%%%%%%%%%%
% Starting values % 
%%%%%%%%%%%%%%%%%%%

% starting value via GTLS and BTLS estimate
FigNum = 1;
ThetaStart = StartPlantModel(data, Sel, ModelVar, IterVar, FigNum);

%%%%%%%%%%%%%%%%%%%%%%%
% Estimation OE model %
%%%%%%%%%%%%%%%%%%%%%%%

[Theta, Cost, smax, smin, wscale] = MIMO_BoxJenkins(data, Sel, ThetaStart, ModelVar, IterVar);


%%%%%%%%%%%%%%%%%%%%%%%%%%
% Cramer-Rao lower bound %
%%%%%%%%%%%%%%%%%%%%%%%%%%

[CRbound, Theta, CovThetan, Thetan, Seln, wscale, TheCond] = MIMO_CR_bound(data, Sel, Theta, ModelVar);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Calculation true, estimated plant transfer functions %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

switch ModelVar.PlantPlane
	case 'z', x.Plant = exp(-sqrt(-1)*2*pi*freq*Ts);
	case 's', x.Plant = sqrt(-1)*2*pi*freq;
	case 'w', x.Plant = (sqrt(-1)*2*pi*freq).^0.5;
end
x.Noise = ones(size(x.Plant));

PolyTrans0 = CalcPolyTrans(Theta0, x);
PolyTrans = CalcPolyTrans(Theta, x);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Calculation true poles, resonance frequencies, %
% damping ratios, time constants                 %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% the covariance has no meaning here
% True plant poles
[CovPoles0, Poles0] = CovRoots(Theta0.A, eye(na+1), Sel.A, PlantPlane, Ts);


%%%%%%%%%%%%%%%%%%%%%
% Plot the results %
%%%%%%%%%%%%%%%%%%%%

figure(FigNum+1)
mm = 0;
for jj = 1:ny
	for ii = 1:nu
		mm = mm+1;
		subplot(ny, nu, mm)
		plot(freq,db(squeeze(PolyTrans0.G(jj,ii,:))), 'k', freq, db(squeeze(PolyTrans.G(jj,ii,:))), 'r', ...
             freq, db(squeeze(PolyTrans0.G(jj,ii,:)-PolyTrans.G(jj,ii,:))), 'k--', ...
             freq, db(squeeze(CRbound.G(jj,ii,:)))/2, 'r--');
        xlabel('Frequency (Hz)')
        ylabel('G (dB)')
	end % ii
end % jj
subplot(ny, nu, ceil(nu/2));
title('true value: black; estim.: red; diff.: black --; CR: red --');
zoom on; shg


figure(FigNum+2)
mm = 0;
for jj = 1:ny
	for ii = 1:nu
		mm = mm+1;
		subplot(ny, nu, mm)
		plot(freq,angle(squeeze(PolyTrans0.G(jj,ii,:)))*180/pi, 'k', ...
             freq, angle(squeeze(PolyTrans.G(jj,ii,:)))*180/pi, 'r');
        xlabel('Frequency (Hz)')
        ylabel('angle(G) ()')
	end % ii
end % jj
subplot(ny, nu, ceil(nu/2));
title('true value: black; estim.: red');
zoom on; shg

% comparison true and estimated resonance frequencies plant: case 1, known reference signal 
disp('Estimates plant model')
disp('Estim. f0 [Hz], std(f0) [Hz], estim. - true [Hz]')
[Theta.plant.poles.freq, CRbound.plant.poles.freq.^0.5, Theta.plant.poles.freq-Poles0.freq]

% comparison true and estimated damping ratios plant: case 1, known reference signal 
disp('Estim. damping, std(damping), estim. - true')
[Theta.plant.poles.damp, CRbound.plant.poles.damp.^0.5, Theta.plant.poles.damp-Poles0.damp]

if PlantPlane == 's'
    disp('Estim. tau [s], std(tau) [s], estim. - true [s]')
    [Theta.plant.poles.time, CRbound.plant.poles.time.^0.5, Theta.plant.poles.time-Poles0.time]    
end % if s-domain plant model
