function Theta = MIMO_ML_ExtractParam(DeltaParam, OldTheta, Sel);
%
% function Theta = MIMO_ML_ExtractParam(DeltaParam, OldTheta, Sel);
%
%		Extracts the parameter update Theta from the variation of the
%		free parameters in DeltaParam
%
%	Output parameter
%
%		Theta		=	new estimate plant, noise, and initial conditions parameters
%						structure with fields 'A', 'B', 'Ig'
%							Theta = struct('A',[],'B',[], 'Ig', [])
%							Theta.A = 1 x (OrderA+1)
%								Theta.A(r) = coefficient a(r-1) of Omega^(r-1) 
%							Theta.B = ny x nu x (OrderB+1)
%								Theta.B(i,j,r) = coefficient b(i,j,r-1) of Omega^(r-1)
%							Theta.Ig = ny x (OrderIg+1)
%								Theta.Ig(i,r) = coefficient ig(i,r-1) of Omega^(r-1) 
%
%	Input parameter
%
%		DeltaParam	=	column vector containing the free model parameters,
%						size: number of free parameters x 1 
%
%		OldTheta	=	previous estimate plant, noise, and initial conditions parameters
%						structure with fields 'A', 'B', 'Ig'
%						OldTheta = struct('A',[],'B',[], 'Ig', [])
%							OldTheta.A = 1 x (OrderA+1)
%								OldTheta.A(r) = coefficient a(r-1) of Omega^(r-1) 
%							OldTheta.B = ny x nu x (OrderB+1)
%								OldTheta.B(i,j,r) = coefficient b(i,j,r-1) of Omega^(r-1)
%							OldTheta.Ig = ny x (OrderIg+1)
%								OldTheta.Ig(i,r) = coefficient ig(i,r-1) of Omega^(r-1) 
%
%		Sel			=	structure with fields 'A', 'B', 'Ig'
%							Sel = struct('A',[],'B',[], 'Ig', [])
%							Sel.A = 1 x (OrderA+1)
%								Sel.A(r) = 1 if coeff. a(r-1) is unknown
%								Sel.A(r) = 0 if coeff. a(r-1) = 0
%							Sel.B = ny x nu x (OrderB+1)
%								Sel.B(i,j,r) = 1 if coeff. b(i,j,r-1) is unknown
%								Sel.B(i,j,r) = 0 if coeff. b(i,j,r-1) = 0
%							Sel.Ig = ny x (OrderIg+1)
%								Sel.Ig(i,r) = 1 if coeff. ig(i,r-1) is unknown
%								Sel.Ig(i,r) = 0 if coeff. ig(i,r-1) = 0
%
%
% Copyright (c) Rik Pintelon, Vrije Universiteit Brussel - dept. ELEC, November 2009
% All rights reserved.
% Software can be used freely for non-commercial applications only.
%

nu = size(OldTheta.B,2);
ny = size(OldTheta.B,1);

% coefficients polynomial A
sumA = sum(Sel.A);
DeltaA = zeros(1,length(Sel.A));
lower = 1;
upper = sumA;
DeltaA(Sel.A == 1) = DeltaParam(lower:upper).';
Theta.A = OldTheta.A + DeltaA;

% coefficients ny x nu matrix polynomial B
Theta.B = zeros(size(OldTheta.B));
for jj = 1:nu
	for ii = 1:ny
		sumB = sum(Sel.B(ii,jj,:));
		DeltaB = zeros(1, 1, length(Sel.B(ii,jj,:)));
		lower = upper + 1;
		upper = lower + sumB - 1;
		DeltaB(1, 1, Sel.B(ii,jj,:) == 1) = DeltaParam(lower:upper).';
		Theta.B(ii,jj,:) = OldTheta.B(ii,jj,:) + DeltaB;
	end % jj
end % ii

% coefficients ny x 1 vector polynomial Ig
Theta.Ig = zeros(size(OldTheta.Ig));
for ii = 1:ny
	sumIg = sum(Sel.Ig(ii,:));
	DeltaIg = zeros(1, length(Sel.Ig(ii,:)));
	lower = upper + 1;
	upper = lower + sumIg - 1;
	DeltaIg(Sel.Ig(ii,:) == 1) = DeltaParam(lower:upper).';
	Theta.Ig(ii,:) = OldTheta.Ig(ii,:) + DeltaIg;
end % ii
